/* ********************************************************************************
Funktion:
ATtiny13a (4.8MHz interner RC-Oszillator) empfngt RAM und ROM Daten von DS18B20
oder DS18S20 im 5 Sekunden-Takt.
Die Daten werden umgerechnet und per I2C ber PCF8574 am LCD1602/2004 angezeigt.
Line1: Anzeige DS18B20 oder DS18S20 sowie 4 Fehlerbits
Line2: Temperatur -55,0 bis +125,0C. 

Beispiel

 DS18B20 F:0000
+020,5C

Taste: ca. 2 Sekunden ID, dann ca. 2 Sekunden Scratchpad-Daten plus CRC Ergebnis anzeigen.
ber Test-Pin bei Bedarf einen Takt ausgeben
*/
; Programm Code weiterentwickelt von Manfred Gabriel  manfred.gabriel@hotmail.de
; auf Bitte von Manfred hier eingestellt
; https://www.mikrocontroller.net/topic/499008
; https://www.mikrocontroller.net/attachment/463169/14_4_T13_I2C_PCF_LCD_MC.asm
;
; Aufgerumt von Falk Brunner, 03/2022
;
; *********************************************************************************
;
;                ATtiny13A
;                 4,8 MHz
;                --------
;               |*       |
;         RESET |1      8| VCC
;   SCL <-- PB3 |2      7| PB2 (SCK)  --< Taste
;   SDA <-> PB4 |3      6| PB1 (MISO) --> Test Pin
;           GND |4      5| PB0 (MOSI) <-> OneWire (DS18x20)
;               |        |
;                --------
;
; Achtung! SCL, SDA und OneWire brauchen externe Pull Up Widerstnde
; I2C ca. 5-10k, OneWire ca. 1,5-5k
; Der DS18x20 muss normal mit VCC versorgt werden,
; parasitre Versorgung wird von der Software nicht untersttzt.
;
; --------------------------------
;   AVR Fuses
;   Standard  | 4,8 MHz RC Oszillator
; --------------------------------
;   SPIEN   0 = 0
;   EESAVE  1 = 1
;   WDTON   1 = 1
;   CKDIV8  0 > 1 <
;   SUT1    1 = 1
;   SUT0    0 = 0
;   CKSEL1  1 > 0 <
;   CKSEL0  0 > 1 <
;----------------------------------------------------------------------------------

.include "tn13adef.inc"
.def    NULL            = R1        ; immer NULL
.def    Divisor         = R15       ; CRC Polynom
; ------------------------------
.def    temp0           = R16       ; 
.def    temp1           = R17       ; 
.def    temp2           = R18       ; 
.def    temp3           = R19       ; 
.def    temp_warte      = R20       ; 
.def    LSB             = R21       ; Temperatur
.def    MSB             = R22       ; Temperatur
.def    CRC             = R23       ; CRC Zwischenspeicher
.def    Fehler          = R24       ; Fehler
.def    Licht           = R25       ; Bit #0 schaltet Licht fr LCD
; Fehlerbits
.equ    OW_RESET_LOW    = 0         ; OneWire RESET Fehler, Bus dauerhaft LOW (Kurzschlu?)
.equ    OW_RESET_HIGH   = 1         ; OneWire RESET Fehler, Bus dauerhaft HIGH, IC reagiert nicht
.equ    CRC_ID          = 2         ; DS18x20 ID CRC Fehler
.equ    CRC_SP          = 3         ; DS18x20 Scratchpad CRC Fehler
;----------------------------------------------------------------------------------
.equ    PORT_OUT        = PORTB     ; Port definieren
.equ    PORT_DDR        = DDRB      ; Datenrichtungsregister fr Port
.equ    PORT_IN         = PINB      ; data Input

.equ    TEST_PIN        = PB1       ; Test Pin, Ausgang
.equ    Taste           = PB2       ; Taste
.equ    SCL             = PB3       ; I2C_SCL
.equ    SDA             = PB4       ; I2C_SDA
;----------------------------------------------------------------------------------
.cseg
.org    0x0000      rjmp RESET      ; 0x0000 Reset Handler
/*
.org    INT0addr    rjmp INT_ERROR  ; External Interrupt 0
.org    PCI0addr    rjmp INT_ERROR  ; External Interrupt Request 0
.org    OVF0addr    rjmp INT_ERROR  ; Timer/Counter0 Overflow
.org    ERDYaddr    rjmp INT_ERROR  ; EEPROM Ready
.org    ACIaddr     rjmp INT_ERROR  ; Analog Comparator
.org    OC0Aaddr    rjmp INT_ERROR  ; Timer/Counter Compare Match A
.org    OC0Baddr    rjmp INT_ERROR  ; Timer/Counter Compare Match B
.org    WDTaddr     rjmp INT_ERROR  ; Watchdog Time-out
.org    ADCCaddr    rjmp INT_ERROR  ; ADC Conversion Complete
; *********************************************************************************
INT_ERROR:
    reti
*/
;**************************************************************************
Version:    .db     "V1.10 ??.03.2022"      ; Falk Brunner
; *********************************************************************************
.include    "wait.asm"                      ; verschiedene Wartefunktionen
;-------------------------------------------
.include    "I2C_Master.asm"                ; I2C-Ansteuerung
.include    "I2C_LCD.asm"                   ; LCD-Ansteuerung ber I2C
;-------------------------------------------
.include    "DS18x20_Read.asm"              ; DS18x20 auslesen
.include    "DS18x20_Temperatur.asm"        ; DS18x20 Temperatur berechnen und ausgeben
.include    "print_data.asm"                ; Ausgabe von ID, Temperaturdaten, DS18x20 Typ und Fehlerbits 
;-------------------------------------------
; *********************************************************************************
RESET:
;----------------------------------------------------------------------------------
    ldi     temp0, LOW(RAMEND)              ; Stack initialisieren
    out     SPL, temp0
    clr     NULL                            ; Konstanten initialisieren 
    ldi     Temp0, CRC_POLYNOM
    mov     Divisor, Temp0
    sbi     PORT_DDR, TEST_PIN              ; PB1 ist Ausgang
    sbi     PORT_OUT, Taste                 ; internen Pull-Up Widerstand einschalten
;-------------------------------------------
; OneWire und I2C sind beide Open Drain Busse!
; Pins bleiben auf Eingang mit externen Pull-Up Widerstnden
; Die PORT-Bits mssen auf 0 stehen und auch immer so bleiben!
; Die I2C und OneWire Funktionen schalten nur die DDR-Bits!
; DDR-Bit = 0; IO-Pin Eingang (HIGH-Z) -> externer Pull Up Widerstand zieht auf HIGH
; DDR-Bit = 1; IO-Pin Ausgang LOW
;-------------------------------------------
    rcall   LCD_init                        ; LCD ber I2C initialisieren
    rcall   print_version
    rcall   warte_5s_taste
;-------------------------------------------
; *********************************************************************************
hauptschleife:
;----------------------------------------------------------------------------------
    rcall   DS18x20_read_id
    rcall   DS18x20_read_scratchpad
    rcall   Temperatur_umrechnen
    rcall   LCD_Clear_Display
    rcall   Temperatur_anzeigen
    rcall   print_DS18x20_type
    rcall   print_DS18x20_error
;-------------------------------------------
    sbi     PORT_IN, TEST_PIN               ; testweise Takt ausgeben, Ausgang invertieren
    rcall   warte_5s_taste                  ; 5s warten, Abbruch mit Taste
    ldi     temp2, 20
    rcall   warte_Nx1ms                     ; einfache Tastenentprellung
;-------------------------------------------
    sbic    PORT_IN, Taste                  ; Taste gedrckt?
    rjmp    hauptschleife                   ; Schleifenende 
; *********************************************************************************
    rcall   LCD_Clear_Display
    rcall   print_family_code
    rcall   warte_2s
    rcall   LCD_Clear_Display
    rcall   print_temp_data
    rcall   warte_2s
    rjmp    hauptschleife
; *********************************************************************************

